<?php
/**
 * @file
 *  API calls for use across all networks 
 */

/**
 * @return
 *  An associative array of all available network names
 */
function user_relationship_locator_networks() {
  $networks = db_select('user_relationship_locator','url')
    ->fields('url', array('netname', 'users'))
    ->execute()
    ->fetchAssoc();
  if (empty($networks)) {
    if (user_access('Administer relationship location')) {
      drupal_set_message('Please enable a network module to provide user data to the relationship locator.');
    }
  }
  return $networks;
}

/**
 * Returns a list of networks a given user is part of 
 * @param $uid
 *  User ID is most likely the current user.
 * @return $memberships
  $memberships = array();
  if($networks = user_relationship_locator_networks()) {
    foreach($networks as $network) {
 *  an array of network names
 */
function user_relationship_locator_network_memberships($uid) {
  $memberships = array();
  if($networks = user_relationship_locator_networks()) {
    foreach($networks as $network) {
      $result = db_select('user_relationship_locator','url')
        ->fields('url', array('netname', 'users'))
        ->condition('netname', $network)
        ->execute();
      foreach($result as $key => $value) {
        if(!empty($value->users)) {
          $users = unserialize($value->users);
          if(in_array($uid,$users)) {
            $memberships[] = $value->netname;
          }
        }
      }
      return $memberships;
    }
  }
}

/**
 * Given a network name return the module name
 */
function user_relationship_locator_network_module($netname) {
  //TODO: this feels um... fragile
  $module_name = 'user_relationship_locator_' . $netname;
  return $module_name;
}

/**
 * Returns a list of user ids who are possible matches
 */
function user_relationship_locator_matches($uid) {
  $max = variable_get('user_relationship_locator_max','');
  $min = variable_get('user_relationship_locator_min','');
  $matches = array();
  
  //don't waste energy querying networks our user doesn't belong to
  if($memberships = user_relationship_locator_network_memberships($uid)) {
    foreach($memberships as $network) {
      $modulename = user_relationship_locator_network_module($network); 
      $network_users = module_invoke($modulename, 'urlocator_user_query', $uid);
      foreach($network_users as $key => $value) {
        $matches[$key] = $key;
      }
    }
  }
  
  //Fill an array with randoms UIDS to fill our block if we don't have enough matches
  //TODO: instead of random this could be pseudo random where it starts only filling random people from in-network then branches out
  $result = db_query('SELECT uid from {users} WHERE uid > 1');
  $allusers = array();
  foreach($result as $value) {
    $allusers[$value->uid] = $value->uid;
  }
  if(count($matches) < $min) {
    if($min <= count($allusers)) {
      $filler = array_rand($allusers,($min - count($matches)));
      $matches = array_merge($matches,$filler);
    } else {
      $matches = array_merge($matches,$allusers);
    }
  }
  //TODO: exclude current relationships
  $matches = user_load_multiple($matches);
  return $matches;
}

/**
 * Create a new network 
 */
function user_relationship_locator_network_create($netname,$users) {
  //TODO: do check here to see if $netname already exists and throw error if does
  //TODO: have the network name automatically be the submodule name? this is necessary
  $users = serialize($users);
  $fields = array('netname' => $netname, 'status' => 1, 'users' => $users); 
  db_insert('user_relationship_locator')->fields($fields)->execute();
}

/**
 * Load an entire network object
 * @param $netname
 *  Network name.
 * @return $network
 *  Network object. 
 */
function user_relationship_locator_network_load($netname) {
}

/**
 * Returns a list of users from a given network 
 * @param $netname
 *  Network name.
 * @return $members
 *  an array of uids 
 */
function user_relationship_locator_network_users($netname) {
}

/**
 * Delete a network 
 */
function user_relationship_locator_network_delete($netname) {
  db_delete('user_relationship_locator')
    ->condition('netname', $netname)
    ->execute();
}

/**
 * Update an existing network's information. 
 * @param $users
 *  Associative array key = uid, value = unique network id for user
 * @return $success 
 *  boolean if update succeeeds 
 */
function hook_urlocator_network_update($netname, $users, $status = 1) {
}

/**
 * UID comparison list from a network.
 * @return $users
 *  an array of possible user id matches to act upon 
 */
function hook_urlocator_user_query($uid) {
}
